﻿/*
 * jQuery Snappy 0.1.7
 *
 * Copyright 2013, Fifty Eggs Ltd
 * Free to use under the MIT license.
 */

(function ($) {
    $.fn.snappy = function (options) {
        /*
         * Default offset calc function
         */
        var offsettop = function (e) {
            return $(e).offset().top;
        };

        var offsetleft = function (e) {
            return $(e).offset().left;
        };
        var offsetbottom = function (e) {
            var el = $(e);
            return (el.offset().top*1.0) + (el.height()*1.0);
        };
        var offsetright = function (e) {
            var el = $(e);
            return (el.offset().left * 1.0) + (el.width() * 1.0);
        };

        var storeOriginal = function (element, pass, currentValue) {
            if (pass === 0) {
                var original = element.data('50ESnappy');
                if (original === undefined) {
                    element.data('50ESnappy', currentValue);
                }
            }
        };
        var resetToOriginal = function (element, pass, adjust) {
            if (pass === 0) {
                var original = element.data('50ESnappy');
                element.css(adjust, original);
            }
        };

        var settings = $.extend({
            "adjust": "padding-top",
            "tolerance": 50,
            "maxpass": 1,
            "measure": offsettop,
            "reset": false
        }, options);

        if (settings.measure == "offsettop") settings.measure = offsettop;
        else if (settings.measure == "offsetleft") settings.measure = offsetleft;
        else if (settings.measure == "offsetbottom") settings.measure = offsetbottom;
        else if (settings.measure == "offsetright") settings.measure = offsetright;

        for (var pass = 0; pass < settings.maxpass; pass++) {
            var positions = [];
            this.each(function () {
                if (settings.reset) resetToOriginal($(this), pass, settings.adjust);
                storeOriginal($(this), pass, $(this).css(settings.adjust).replace("px", "") * 1);
                var offset = settings.measure(this);
                var thisEntry = {
                    "offset": offset,
                    "element": this
                };
                for (var i = 0; i < positions.length; i++) {
                    var otherEntry = positions[i];
                    var diff = Math.abs(otherEntry.offset - offset);
                    if ((diff > 0) && (diff < settings.tolerance)) {
                        if (otherEntry.offset < offset) {
                            var otherElement=$(otherEntry.element);
                            var othercv = otherElement.css(settings.adjust).replace("px", "") * 1;
                            storeOriginal(otherElement, pass, othercv);
                            otherElement.css(settings.adjust, othercv + diff);
                            otherEntry.offset = offset;
                        }
                        else {
                            var thisElement=$(thisEntry.element);
                            var thiscv = thisElement.css(settings.adjust).replace("px", "") * 1;
                            thisElement.css(settings.adjust, thiscv + diff);
                            offset = otherEntry.offset;
                        }
                    }
                }
                positions.push(thisEntry);
            });
        }
        return this;
    };
}(jQuery));